/*
 * packetmodeller/typedef.c
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <string.h>
#include "typedef.h"

/**
 *  Retrieve MAC addresses from Ethernet frame
 */
int
get_mac(mac_addr *source_mac, mac_addr *dest_mac, const RAWPACKET *packet)
{
    const struct mac_hdr *header = (const struct mac_hdr *) packet;

    memcpy(dest_mac, &(*header).mac_dest, sizeof(mac_addr));
    memcpy(source_mac, &(*header).mac_src, sizeof(mac_addr));

    return 0;
}

/**
 *  Return IP packet type (0 = IPv4, 6 = ARP, -1 = unknown)
 */
int get_packet_type(char *type_string, const RAWPACKET *packet)
{
    int i, type;
    const int types_nr = 2;
    const int valid_types[2] = { 0, 6 };
    const char *type_strings[types_nr + 1] = { "unknown", "IPv4", "ARP " };

    type = ((struct mac_hdr *) packet)->pkt_type[1];
    for (i = 0; i < types_nr; i++)
    {
        if (type == valid_types[i])
        {
            strcpy(type_string, type_strings[i + 1]);
            return type;
        }
    }
    strcpy(type_string, type_strings[0]);

    return -1;
}
