/*
 * wlanmodel/message.h
 *
 * Copyright (C) 2008-2009 Reto Gantenbein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __MESSAGE_H
#define __MESSAGE_H

#include <net/ethernet.h>
#include <netinet/in.h>
#include "typedef.h"

/**
 *  Message types
 */

/** Message from node -> model */
#define NODE_MSG                0
#define NODE_PREAMBLE_SIZE      (sizeof(struct msg_header) + \
                                2 * sizeof(unsigned char)) + \
                                sizeof(uint16_t)

/** Message from model -> node */
#define MODEL_MSG               1
#define MODEL_PREAMBLE_SIZE     sizeof(struct msg_header)

/** Register message to node */
#define REGISTER_MSG            2
#define REGISTER_MSG_SIZE       (sizeof(struct msg_header) + \
                                2 * sizeof(uint16_t) + \
                                HOSTNAME_SIZE + \
                                sizeof(in_addr) + \
                                ETH_ALEN * sizeof(uint8_t))

/** Acknowledge message from model */
#define ACK_MSG                 3
#define ACK_MSG_SIZE            (sizeof(struct msg_header) + \
                                sizeof(unsigned char))

/**
 *  Message header
 *  @var type: message type
 *  @var id: sender identification
 */
struct msg_header {
    unsigned char type;
    host_id id;
};

/**
 *  Node message
 *  @var channel: wireless channel of sender node
 *  @var transmit_power: transmit power of sender node
 *  @var wlan_pkt: raw Ethernet packet
 */
struct node_msg {
    msg_header header;
    unsigned char channel;
    unsigned char transmit_power;
    uint16_t pkt_length;
    RAWPACKET wlan_pkt;
};

/**
 *  Model message
 *  @var wlan_pkt: raw Ethernet packet
 */
struct model_msg {
    msg_header header;
    RAWPACKET wlan_pkt;
};

/**
 *  Register message
 *  @var msg_id: message identification
 *  @var name: node hostname
 *  @var ip: node IP address
 *  @var port: node listen port
 */
struct register_msg {
    msg_header header;
    uint16_t msg_id;
    char name[HOSTNAME_SIZE];
    in_addr ip;
    uint16_t port;
    uint8_t virt_mac[ETH_ALEN];
};

/**
 *  Acknowledge message
 *  @var msg_id: message identification (same as register msg_id)
 */
struct ack_msg {
    msg_header header;
    unsigned char msg_id;
};

#endif
